/*
   ITU-T G.729A Speech Coder    ANSI-C Source Code
   Version 1.1    Last modified: September 1996

   Copyright (c) 1996,
   AT&T, France Telecom, NTT, Universite de Sherbrooke
   All rights reserved.
*/

/*-----------------------------------------------------------------*
 * Main program of the G.729a 8.0 kbit/s decoder.                  *
 *                                                                 *
 *    Usage : decoder  synthetic_file  synth_file                  *
 *                                                                 *
 *-----------------------------------------------------------------*/

#include <stdlib.h>
#include <stdio.h>

#include "typedef.h"
#include "basic_op.h"
#include "ld8a.h"

 Word16 bad_lsf;        /* bad LSF indicator   */

/*
   This variable should be always set to zero unless transmission errors
   in LSP indices are detected.
   This variable is useful if the channel coding designer decides to
   perform error checking on these important parameters. If an error is
   detected on the  LSP indices, the corresponding flag is
   set to 1 signalling to the decoder to perform parameter substitution.
   (The flags should be set back to 0 for correct transmission).
*/

/*
 * Questa funzione fa l'operazione inversa della 'info_synt' descritta
 * in 'coder.c'.
*/

void info_extract( unsigned char payload[10], Word16 parm[PRM_SIZE+1])
{
 
 int i;
 unsigned short buff[10];
	
 for(i=0;i<10;i++)
  buff[i] = payload[i];
	
 parm[1]  = buff[0];
 parm[2]  = buff[1]+((buff[2]&192)<<2);
 parm[3]  = (buff[2]&63)+(buff[3]&192);
 parm[4]  = ((buff[3]&32)>>5)&1;
 parm[5]  = (buff[3]&31)+(buff[4]<<5);
 parm[6]  = ((buff[5]&240)>>4)&15;
 parm[7]  = (buff[5]&15)+(((buff[6]&224)>>1)&112);
 parm[8]  = buff[6]&31; 
 parm[9]  = buff[7]+((buff[8]&248)<<5);
 parm[10] = (buff[8]&7)+(((buff[9]&128)>>4)&8);
 parm[11] = buff[9]&127;
 
}


/*-----------------------------------------------------------------*
 *            Main decoder routine                                 *
 *-----------------------------------------------------------------*/
 
int main(int argc, char *argv[] )
{
  Word16  synth_buf[L_FRAME+M], *synth; /* Synthesis                   */
  Word16  parm[PRM_SIZE+1];             /* Synthesis parameters        */
  Word16  Az_dec[MP1*2];                /* Decoded Az for post-filter  */
  Word16  T2[2];                        /* Pitch lag for 2 subframes   */
  unsigned char   payload[10];


  Word16  i, frame;
  FILE   *f_syn, *f_synthetic;

  printf("\n");
  printf("************   G.729a 8.0 KBIT/S SPEECH DECODER  ************\n");
  printf("\n");
  printf("------------------- Fixed point C simulation ----------------\n");
  printf("\n");
  printf("-----------------          Version 1.1        ---------------\n");
  printf("\n");

   /* Passed arguments */

   if ( argc != 3)
     {
        printf("Usage :%s synthetic_file  outputspeech_file\n",argv[0]);
        printf("\n");
        printf("Format for synthetic_file:\n");
        printf("   Like bit packing proposed in 'draft-ietf-avt-profile'.\n");
        printf("\n");
        printf("Format for outputspeech_file:\n");
        printf("  Synthesis is written to a binary file of 16 bits data.\n");
        exit( 1 );
     }

   /* Open file for synthesis */

   if( (f_synthetic = fopen(argv[1],"rb") ) == NULL )
     {
        printf("%s - Error opening file  %s !!\n", argv[0], argv[1]);
        exit(0);
     }

   if( (f_syn = fopen(argv[2], "wb") ) == NULL )
     {
        printf("%s - Error opening file  %s !!\n", argv[0], argv[2]);
        exit(0);
     }

   printf("Input synthetic file  :   %s\n",argv[1]);
   printf("Synthesis speech file :   %s\n",argv[2]);

/*-----------------------------------------------------------------*
 *           Initialization of decoder                             *
 *-----------------------------------------------------------------*/

  for (i=0; i<M; i++) synth_buf[i] = 0;
  synth = synth_buf + M;

  bad_lsf = 0;          /* Initialize bad LSF indicator */
  Init_Decod_ld8a();
  Init_Post_Filter();
  Init_Post_Process();


/*-----------------------------------------------------------------*
 *            Loop for each "L_FRAME" speech data                  *
 *-----------------------------------------------------------------*/

  frame = 0; 
  while( fread(payload, sizeof(char), 10, f_synthetic) == 10)
  {
    		
    printf("Frame =%d\r", frame++);
    
    info_extract(payload, parm); // bit depacketization
     
    /* the hardware detects frame erasures by checking if all bits
       are set to zero
     */
    parm[0] = 0;           /* No frame erasure */

    /* check pitch parity and put 1 in parm[4] if parity error */

    parm[4] = Check_Parity_Pitch(parm[3], parm[4]);

    Decod_ld8a(parm, synth, Az_dec, T2);

    Post_Filter(synth, Az_dec, T2);        /* Post-filter */

    Post_Process(synth, L_FRAME);

    fwrite(synth, sizeof(short), L_FRAME, f_syn);

  }
  return(0);
}

